package main

/*
	validate / sanity check / consistancy check the data

	here is some code to do smart things like:

	* check mac addresses are unique
	* check uuid's are unique
	* double check filenames are unique
	* return a unique mac address
	* return a unique uuid

*/

import (
	"os"
	"path/filepath"

	"github.com/google/uuid"

	pb "go.wit.com/lib/protobuf/virtbuf"
	"go.wit.com/log"
)

// will make sure the mac address is unique
func checkUniqueMac(mac string) bool {
	for _, d := range me.cluster.Droplets {
		for _, n := range d.Networks {
			if n.Mac == mac {
				log.Info("duplicate MAC", n.Mac, "in droplet", d.Hostname)
				return false
			}
		}
	}
	return true
}

func addClusterFilepath(dir string) {
	var found bool = false
	for _, d := range me.dirs {
		if d == dir {
			// found dir
			found = true
			break
		}
	}
	if !found {
		me.dirs = append(me.dirs, dir)
	}
}

func checkUniqueFilenames() bool {
	var ok bool = true
	var disks map[string]string
	disks = make(map[string]string)

	for _, d := range me.cluster.Droplets {
		for _, disk := range d.Disks {
			filename := disk.Filename
			if _, ok := disks[filename]; ok {
				/*
				if argv.IgnDisk {
					log.Info("ignore dup disk", filename, disks[filename], d.Hostname)
				} else {
				}
				*/
				log.Info("file", filename, "on droplet", disks[filename])
				log.Info("file", filename, "on droplet", d.Hostname)
				log.Info("duplicate disk names (--xml-ignore-disk to ignore)")
				ok = false
			}
			disks[filename] = d.Hostname
		}
	}
	if ok {
		log.Println("validated okay: no duplicate disk images")
	}
	return ok
}

func checkDiskFilenames() []*pb.Event {
	var alle []*pb.Event

	for _, d := range me.cluster.Droplets {
		for _, disk := range d.Disks {
			filename := disk.Filename
			filebase := filepath.Base(filename)
			dir := filepath.Dir(filename)
			if disk.Filename != filebase {
				// update filename
				e := NewChangeEvent(d, "Disk.Filename", disk.Filename, filebase)
				alle = append(alle, e)
				disk.Filename = filebase
			}
			if dir == "." {
				continue
			}
			if dir == "" {
				continue
			}
			if disk.Filepath != dir {
				// update filename
				e := NewChangeEvent(d, "Disk.Filepath", disk.Filepath, dir)
				alle = append(alle, e)
				disk.Filepath = dir
			}
		}
	}
	return alle
}

func checkDroplets(dump bool) bool {
	// uuid map to check for duplicates
	var umap map[string]string
	umap = make(map[string]string)

	// mac address map to check for duplicates
	var macs map[string]string
	macs = make(map[string]string)

	for _, d := range me.cluster.Droplets {
		// Generate a new UUID
		if d.Uuid == "" {
			u := uuid.New()
			d.Uuid = u.String()
		}

		// seconds, ok := timeZone[tz]; ok {
		if _, ok := umap[d.Uuid]; ok {
			// UUID already exists
			log.Info("duplicate UUID", d.Uuid, umap[d.Uuid])
			log.Info("duplicate UUID", d.Uuid, d.Hostname)
			os.Exit(-1)
		}
		umap[d.Uuid] = d.Hostname

		for _, n := range d.Networks {
			// log.Println("network:", n.Mac, d.Uuid, d.Hostname)
			if _, ok := macs[n.Mac]; ok {
				// UUID already exists
				log.Info("duplicate MAC", n.Mac, macs[n.Mac], umap[macs[n.Mac]])
				log.Info("duplicate MAC", n.Mac, d.Hostname)
				os.Exit(-1)
			}
			macs[n.Mac] = d.Uuid
		}
	}
	log.Println("validated okay: no duplicate MAC addr")
	log.Println("validated okay: no duplicate UUID")

	if dump {
		for u, hostname := range umap {
			log.Println("uuid:", u, "hostname:", hostname)
		}

		for mac, uuid := range macs {
			log.Println("mac:", mac, "uuid", uuid, "hostname:", umap[uuid])
		}
	}

	return false
}
